@@ -36,6 +36,8 @@
 #include "debug.h"
 #include "i18n.h"
 
+static char* _find_file_in_path(const char* files, gboolean relative);
+static char* _find_file_in_path_list(char** binaries, gboolean relative);
 
 short strtosh (const char *str) {
   long tmp;
@@ -85,7 +87,7 @@ char *file_in_dir (const char *dir, const char *file) {
   char *res, *tmp;
   int need_slash = 0;
 
-  if (!dir || dir[0] == '\0')			/* dir "" is current dir */
+  if (!dir || dir[0] == '\0') /* dir "" is current dir */
     return (file)? g_strdup (file) : NULL;
 
   if (!file)
@@ -94,7 +96,7 @@ char *file_in_dir (const char *dir, const char *file) {
   if (dir[strlen (dir) - 1] != G_DIR_SEPARATOR)
     need_slash = 1;
 
-  tmp = res = g_malloc (strlen (dir) + strlen (file) + need_slash + 1);
+  tmp = res = g_malloc0 (strlen (dir) + strlen (file) + need_slash + 1);
 
   strcpy (tmp, dir);
   tmp += strlen (dir);
@@ -180,6 +182,9 @@ GList *dir_to_list (const char *dirname,
   GList *list = NULL;
   char *str;
 
+  if(!dirname)
+    return NULL;
+
   directory = opendir (dirname);
   if (directory == NULL)
     return NULL;
@@ -518,13 +523,34 @@ const char* bool2str(int i)
  */
 char* find_file_in_path(const char* files)
 {
-    char** binaries = NULL;
+  return _find_file_in_path(files, FALSE);
+}
+
+char* find_file_in_path_relative(const char* files)
+{
+  return _find_file_in_path(files, TRUE);
+}
+
+char* find_file_in_path_list(char** files)
+{
+  return _find_file_in_path_list(files, FALSE);
+}
+
+char* find_file_in_path_list_relative(char** files)
+{
+  return _find_file_in_path_list(files, TRUE);
+}
+
+/**
+  @param binaries NULL terminated list of strings
+ */
+static char* _find_file_in_path_list(char** binaries, gboolean relative)
+{
     char* path = NULL;
     int i = 0, j = 0;
     char** directories = NULL;
     char* found = NULL;
 
-    binaries = g_strsplit(files,":",0);
     path = getenv("PATH");
 
     if(!binaries) return NULL;
@@ -545,7 +571,7 @@ char* find_file_in_path(const char* files)
 	    {
 		// If directory name is blank, don't add a / - happens if
 		// a complete path was passed
-		if (strlen(directories[j]))
+		if (!relative && strlen(directories[j]))
   		  found = g_strconcat(directories[j],"/",binaries[i],NULL);
 		else
 		  found = g_strdup(binaries[i]);
@@ -555,77 +581,107 @@ char* find_file_in_path(const char* files)
 	}
     }
 
+    g_strfreev(directories);
+
+    return found;
+}
+
+static char* _find_file_in_path(const char* files, gboolean relative)
+{
+    char** binaries = NULL;
+    char* found = NULL;
+
+    binaries = g_strsplit(files,":",0);
+
+    found = _find_file_in_path_list(binaries, relative);
+
     g_strfreev(binaries);
 
     return found;
 }
 
-// find a directory inside another, prefer matching case otherwise search case
-// insensitive
-// returns name of found directory or duplicate of game, must be freed manually
+/** find a directory inside another, prefer matching case otherwise search case insensitive
+ * @param basegamedir where to search
+ * @param game directory to search for
+ * @param match_result output: 0 = not found, 1 = exact match, 2 = differnet case match
+ * @returns name of found directory or NULL, must be freed manually
+ */
 char *find_game_dir (const char *basegamedir, const char *game, int *match_result)
 {
-  DIR *dp;
-  struct dirent *ep;
-  struct stat buf;
-  char *path;
-  char *temp;
+  DIR *dp = NULL;
+  struct stat buf = {0};
+  char *path = NULL;
+  char* ret = NULL;
+  int result = 0;
 
-  *match_result = 0; // No match, unless changed below
-  
-  if(!game)
-    return g_strdup("");
+  if(!game || !basegamedir)
+    return NULL;
 
-  if(!basegamedir)
-    return g_strdup(game);  
-
-  debug( 1, "Looking for subdir/symlink %s in %s", game, basegamedir);
+  debug( 3, "Looking for subdir %s in %s", game, basegamedir);
   
   // Look for exact match
-  // Looks for file, not specifically a subdir or symlink - good enough
-  // FIXME: Should never be a symlink - lstat would give a symlink, not stat
   path = file_in_dir (basegamedir, game);
-  if (!stat (path, &buf)) {
-    if ( (S_ISDIR(buf.st_mode) == 1) || (S_ISLNK(buf.st_mode)) ) {
-      // directory found
-      g_free (path);
-      debug( 1, "Found exact match for subdir/symlink %s in %s", game, basegamedir);
-      *match_result = 1; // Exact match
-      return g_strdup (game);
-    }
+  if (!stat (path, &buf) && S_ISDIR(buf.st_mode))
+  {
+    debug( 3, "Found exact match for subdir %s in %s", game, basegamedir);
+    result = 1; // Exact match
+    ret = g_strdup(game);
   }
   g_free (path);
-  debug( 1, "Did not find exact match for subdir/symlink %s in %s", game, basegamedir);
 
   // Did not find exact match, perform search
-  debug( 1, "Searching for subdir/symlink %s in %s ignoring case", game, basegamedir);
-  dp = opendir (basegamedir);
-  if (dp != NULL)
+  if(!ret)
+  {
+    debug( 3, "Did not find exact match for subdir %s in %s", game, basegamedir);
+    debug( 3, "Searching for subdir %s in %s ignoring case", game, basegamedir);
+
+    dp = opendir (basegamedir);
+    if(!dp)
     {
-      while ((ep = readdir (dp))) {
-	stat(ep->d_name, &buf);
-        if ( (S_ISDIR(buf.st_mode) == 1) || (S_ISLNK(buf.st_mode)) )
-	  if (strcmp(ep->d_name,".") && strcmp(ep->d_name,".."))
-	  {
-	    if (!strcasecmp(ep->d_name,game)) {
-              debug( 1, "Found subdir/symlink %s in %s that matches %s", ep->d_name, basegamedir,
-                                                                    game);
-              *match_result = 2; // Different case match
-	      temp = g_strdup (ep->d_name);
-	      closedir (dp);
-	      return (temp);
-	      break;
-	    }
-  	  }
+      debug( 3, "Could not open base directory %s!", basegamedir);
+    }
+    else
+    {
+      struct dirent *ep;
+      while ((ep = readdir (dp)))
+      {
+	char* name = ep->d_name;
+
+	if (!strcmp(name, ".") || !strcmp(name,".."))
+	  continue;
+
+	path = file_in_dir(basegamedir, name);
+	
+	if(stat(path, &buf) == -1)
+	{
+	  g_free(path);
+	  continue;
+	}
+	
+	g_free(path);
+	
+	if (!S_ISDIR(buf.st_mode))
+	  continue;
+
+	if (!g_strcasecmp(name, game))
+	{
+	    debug( 3, "Found subdir %s in %s that matches %s", ep->d_name, basegamedir, game);
+	    result = 2; // Different case match
+	    ret = g_strdup (name);
+	    break;
+	}
       }
       closedir (dp);
-      debug( 1, "Could not find any match for subdir/symlink %s in %s.  Returning %s",game, 
-                                                                basegamedir, game);
     }
-  else
-    debug( 1, "Could not open base directory %s!!", basegamedir);
+  }
 
-  return g_strdup (game);
+  if(!ret)
+    debug( 3, "Could not find any match for subdir %s in %s.",game, basegamedir);
+
+  if(match_result)
+    *match_result = result;
+
+  return ret;
 }
 
 /** sort list and remove duplicates
@@ -673,83 +729,152 @@ GSList* slist_sort_remove_dups(GSList* list, GCompareFunc compare_func, void (*u
   return list;
 }
 
+/** \brief determine directory for game binary
+ *
+ * Extracts the path from path using the following rules:
+ * - If path is a symlink:
+ *   - If pointed to file contains '..', just stop, otherwise strip filename and
+ *     store as directory
+ *   - If there is no /'s in the pointed to file, use the original path instead and
+ *     strip filename and store as directory
+ * 
+ * - If path is not a symlink:
+ *   - strip filename and store as directory
+ *
+ * Path can be either a file or a directory
+ *
+ * Examples:
+ *
+ * path:	/usr/bin/quake2 symlink to /games/quake2/quake2
+ * result:	/games/quake2/
+ *
+ * path:	/usr/bin/quake symlink to ../../games/quake2/quake2
+ * result:	(stops - leaves as-is)
+ *
+ * path:	/games/quake2/quake2
+ * result:	/games/quake2/
+ *
+ * path:	quake2
+ * result:	search $PATH for binary then use above rules
+ *
+ * path:	~/bin/quake2
+ * result:	expand tilde then use above rules
+ *
+ * @returns game direcory or NULL. must be freed
+ */
+
 char* resolve_path(const char* path)
 {
-  // Extracts the path from path using the following rules:
-  // - If path is a symlink:
-  //   - If pointed to file contains '..', just stop, otherwise strip filename and
-  //     store as directory
-  //   - If there is no /'s in the pointed to file, use the original cmd_entry instead and
-  //     strip filename and store as directory
-  // 
-  // - If path is not a symlink:
-  //   - strip filename and store as directory
-  //
-  // Path can be either a file or a directory
-  //
-  // Examples:
-  //
-  // cmd_entry:		/usr/bin/quake2 symlink to /games/quake2/quake2
-  // result dir:	/games/quake2/
-  //
-  // cmd_entry:		/usr/bin/quake symlink to ../../games/quake2/quake2
-  // result dir:	(stops - leaves as-is)
-  //
-  // cmd_entry:		/games/quake2/quake2
-  // result dir:	/games/quake2/
-  //
-  // cmd_entry:		quake2
-  // result dir:	(stops - leaves as-is)
-  //
   
-  struct stat buf;
+  struct stat statbuf;
   int length = 0;
-  char buf2[256];
+  char buf[256];
   char *ptr = NULL;
   char *dir = NULL;
+  char* tmp = NULL;
   
-  if (strcmp (path, "")) {
-    lstat(path, &buf);
-    if ( S_ISLNK(buf.st_mode) == 1) {
-      // Grab directory from sym link of cmd_entry
-      
-      debug(2, "path is a sym link");    
+  if(!path || !*path)
+    return NULL;
 
-      length = readlink (path, buf2, 255);
-
-      if (length){
-        buf2[length]='\0';
-
-        if(buf2[length-1] == '/')
-          buf2[length-1] = '\0';
-        
-        ptr = strrchr(buf2, '/');
-        
-        if (ptr) {	    			// contains a / so pull from symlink
-          if (!strstr(buf2,"..")) {		// don't bother if it's got any ..'s in it
-            dir = g_strndup(buf2, ptr-buf2+1);
-          }
-        }
-        else {        				// no / so pull from cmd_entry instead
-          ptr = strrchr(path, '/');
-          if (ptr) {      			// contains a /
-            dir = g_strndup(path, ptr-path+1);
-          }
-        }        
-      }
-    }
-    else {
-      // Grab directory from cmd_entry
-      
-      debug(2,"path is NOT a sym link");
-    
-      ptr = strrchr(path, '/');
-  
-      if (ptr) {      				// contains a /
-        dir = g_strndup(path, ptr-path+1);
-      }
-    }  
+  if(path[0] == '~')
+  {
+    tmp = expand_tilde(path);
+    path = tmp;
   }
+  else if(path[0] != '/')
+  {
+    tmp = find_file_in_path(path);
+    if(!tmp)
+    {
+      debug(3, "%s not found in $PATH", path);
+      return NULL;
+    }
+    path = tmp;
+  }
+
+  if(lstat(path, &statbuf) == -1)
+  {
+    debug(3, "lstat on %s failed", path);
+    g_free(tmp);
+    return NULL;
+  }
+  
+  if ( S_ISLNK(statbuf.st_mode) == 1)
+  {
+    // Grab directory from sym link of cmd_entry
+    
+    debug(3, "path is a sym link");    
+
+    length = readlink (path, buf, sizeof(buf) - 1);
+
+    if (length > 0)
+    {
+      buf[length]='\0';
+
+      if(buf[length-1] == '/')
+	buf[length-1] = '\0';
+      
+      ptr = strrchr(buf, '/');
+      
+      if (ptr) {	    			// contains a / so pull from symlink
+	if (!strstr(buf,"..")) {		// don't bother if it's got any ..'s in it
+	  dir = g_strndup(buf, ptr-buf+1);
+	}
+      }
+      else {        				// no / so pull from path instead
+	ptr = strrchr(path, '/');
+	if (ptr) {      			// contains a /
+	  dir = g_strndup(path, ptr-path+1);
+	}
+      }        
+    }
+  }
+  else
+  {
+    // Grab directory from cmd_entry
+    
+    debug(3,"path is NOT a sym link");
+  
+    ptr = strrchr(path, '/');
+
+    if (ptr) // contains a /
+    {
+      gboolean dir_is_path = FALSE;
+      char* PATH = getenv("PATH");
+
+      // ignore direcory if it is in $PATH. It doesn't make sense to suggest
+      // /usr/bin as game direcory
+      if(PATH)
+      {
+	int i;
+	char** directories = g_strsplit(PATH,":",0);
+	char* basedir = g_strndup(path, ptr-path);
+
+	for(i=0; directories[i]; ++i)
+	{
+	  if(!strcmp(directories[i], basedir))
+	  {
+	    dir_is_path = TRUE;
+	    break;
+	  }
+	}
+
+	g_strfreev(directories);
+	g_free(basedir);
+      }
+
+      if(!dir_is_path)
+      {
+	dir = tmp;
+	tmp = NULL;
+      }
+      else
+	debug(3, "found directory %s in $PATH, ignoring", tmp);
+    }
+  }  
+
+  g_free(tmp);
+
   return dir;
 }
 
