@@ -26,6 +26,7 @@
 #include <arpa/inet.h>	/* struct in_addr */
 #include <time.h>	/* time_t */
 
+#include <gtk/gtk.h>
 #include <glib.h>
 
 #define RC_DIR		".qf"
@@ -99,6 +100,7 @@ enum server_type {
   Q3_SERVER,
   WO_SERVER,
   WOET_SERVER,
+  DOOM3_SERVER,
   EF_SERVER,
   H2_SERVER,
   HW_SERVER,
@@ -123,6 +125,7 @@ enum server_type {
   SAS_SERVER,
   BF1942_SERVER,
   GPS_SERVER,
+  JK3_SERVER,
   UNKNOWN_SERVER
 };
 
@@ -237,8 +240,14 @@ struct master {
   GSList *masters;
   
   enum master_query_type master_type;
+
+  /** private */
+  char* _qstat_master_option; // optional override from games[type].qstat_master_option
 };
 
+char* master_qstat_option(struct master* m);
+void master_set_qstat_option(struct master* m, const char* opt);
+
 extern	time_t xqf_start_time;
 extern char* xqf_PACKAGE_DATA_DIR;
 extern char* xqf_LOCALEDIR;
@@ -259,7 +268,8 @@ int compare_qstat_version ( const char* have, const char* expected );
 int start_prog_and_return_fd(char *const argv[], pid_t *pid);
 int check_qstat_version(void);
 
-extern	void play_sound (const char *sound, const int override);
+void play_sound (const char *sound, gboolean override);
+void play_sound_with (const char* player, const char *sound, gboolean override);
 
 extern	void reset_main_status_bar(void);
 
@@ -270,5 +280,8 @@ extern int event_type;
 extern int dontlaunch;
 
 extern	void refresh_source_list (void);
+extern void update_source_callback (GtkWidget *widget, gpointer data);
+extern void refresh_n_server(GtkWidget * button, gpointer *data);
+extern void stop_callback (GtkWidget *widget, gpointer data);
 
 #endif /* __XQF_H__ */
